import { GraphQLClient } from 'graphql-request/build/esm';
import { setDefaultOptions, format } from 'date-fns';
import { enUS } from 'date-fns/locale';
import { RequestConfig } from 'graphql-request/build/esm/types';
import { State } from './state';
import * as queries from './graphql/queries';
import { FetchAccountsResponse } from '~graphql/models';

setDefaultOptions({ locale: enUS });

const apiEndpoint = 'https://netbanki.landsbankinn.is/services/accounts/$accountID/transactions';

export enum TransactionFormat { JSON };
export enum OutputFormat { CSV };

export interface DownloadTransactionsRequest {
    accountNumber: string;
    dateFrom: Date;
    dateTo: Date;
    outputFormat: OutputFormat;
}

export interface RawTransaction {
    //txn id?
    id: string;
    //often null
    accountNumber: string | null;
    amount: Number;
    currencyCode: string;
    //ISO date
    bookingDate: string;
    //ISO date
    valueDate: string;

    balanceAfterTransaction: Number;

    displayText: string;

    //how do these relate to id?
    reference: string;
    referenceNumber: string;

    //how does it relate to the booking/value dates?
    //exact time of txn/
    timestamp: string;

    textCodeDescription: string;
    payerDescription: string;

    // e.g. Utteky med debitkorti
    description: String;

    batchID: string;

    // only for individuals?
    recipientKennitala: string;

    recipientName: string;
    //probably some kind of txn category (central bank?)
    transactionCodekey: string;

    bankBranch: string;
    debitCardWarrantyNumber: string | null;

    //currency to ISK exchange rate
    exchangeRate: Number;

    // what is displayed to the user in the table
    customerDescription: string;

    //don't know what this could be.
    details: null;

}

//Sat Jan 03 2023
const formatDate = (date: Date) => format(date, 'ccc LLL dd yyyy');

function getUrl(accountNumber: string, dateFrom: Date, dateTo: Date) {
    const url = `https://netbanki.landsbankinn.is/services/accounts/${accountNumber}/transactions/`;
    const from = encodeURIComponent(formatDate(dateFrom));
    const to = encodeURIComponent(formatDate(dateTo));
    const params = `?from=${from}&to=${to}`
    return url + params;
}

export async function downloadTransactions(req: DownloadTransactionsRequest): Promise<Array<RawTransaction>> {
    const url = getUrl(req.accountNumber, req.dateFrom, req.dateTo);
    const options: RequestInit = {
        method: "GET",
        credentials: "include",
        cache: "no-cache",
        referrerPolicy: "strict-origin-when-cross-origin",
        headers: {
            "Accept": "application/json",
            "Content-Type": "application/json",
        },
    };

    const result = await fetch(url, options);
    if (result.ok) {
        return await result.json();
    } else {
        //attempt to read json.
        const error = await result.json();
        if (error.hasOwnProperty('message')) {
            console.error(error['message']);
        }
        console.error(result);
        return new Array();
    }
}

export async function fetchAccounts(): Promise<FetchAccountsResponse> {
    const state = await State.current;
    const options: RequestConfig = {
        mode: "cors",
        credentials: "include",
        cache: "no-cache",
        referrerPolicy: "strict-origin-when-cross-origin",
        headers: {
            apikey: state.apiKey,
            authorization: state.gqlAuthToken,
            "rsa-fingerprint": state.rsaFingerprint
        },
    };

    const graphQLClient = new GraphQLClient('https://graphql.landsbankinn.is/v2', options);

    const variables = {
        "first": 100,
        "filter": {
            "includeClosed": true
        },
    };

    const data: FetchAccountsResponse = await graphQLClient.request(queries.fetchAccounts, variables);
    //TODO transform janky graphql response into nicer data structure for us.
    return data;
}
